/**
 * MultisearchServlet is the client-side of the program. Users
 * can interact with Grid Information Retrieval via this
 * servlet. This can also be run from the command line with
 * Multisearch. (Servlet run with Tomcat.)
 * <p>
 * DATE CREATED: 18 June 2007
 * LAST UPDATED: 10 August 2008
 *
 * @author Kylie McCormick
 * @version 3.0
 */
package edu.arsc.multisearch;

import javax.servlet.http.*;
import java.io.*;
import java.util.StringTokenizer;
import javax.servlet.*;
import edu.arsc.multisearch.merge.*;
import edu.arsc.multisearch.client.*;import edu.arsc.multisearch.select.*;
import java.util.ArrayList;

public class MultisearchServlet extends HttpServlet {

    /** PrintWriter object for servlet. */
    public static PrintWriter web; 

    /** Multisearch Object to use to search. */
    //private Multisearch search = null;

    //some non-javadoc strings
    String queryString;
    String algorithmString;
    String viewValue;
    String restrictionString;

    String inputString;
    String outputString;
    String limitString;

    /**
     * Overwrites HttpServlet's function and prints XHTML
     * and text to the appropriate screen. Provides a simple
     * Query grabber using HTML Forms.
     *
     * @param HttpServletRequest
     * @param HttpServletResponse
     * @throws ServletException      when there are issues with connection/server
     * @throws IOException           when there are issuses with user input
     */
    public void doGet(HttpServletRequest request, HttpServletResponse response) 
	throws ServletException, IOException {

	//get the screen to print to
	response.setContentType("text/html");
	web = response.getWriter();

	//the first XHTML
	printTop();

	//commands that make the servlet tick
	queryString = request.getParameter("sendQuery");
	algorithmString  = request.getParameter("Algorithm");
	viewValue = request.getParameter("Verbose");
	restrictionString = request.getParameter("Restriction");
	inputString = request.getParameter("InputFiles") + "/";
	outputString = request.getParameter("OutputFormat");
	limitString = request.getParameter("Limit");
	
	//modifiable HTML
	//top if is when nothing has yet been submitted
	if(queryString == null || queryString.trim().equals("")) {

	    web.println("<h1>Multisearch Servlet Front-End</h1>");

	    web.println("<p>Welcome to the Arctic Region Supercomputing Center's Multisearch Frontend Servlet! This reflects the new Multisearch in 2008. The tools used for this project include Axis, Tomcat, OGSA-DAI (for backends), and Hadoop.</p>");

	    web.println("<p>Multisearch searches over smaller backends instead of over a huge collection, as most search engines do.</p>");

	    web.println("<p>The first drop-down box asks for a <b>merge algorithm</b>. This allows you to choose how the different sets are compared to one another and put together. The second drop-down box asks for a <b>restriction algorithm</b>, which selects various servers for your query and searches only those, to improve performance. The final drop-down box asks for a <b>view style</b>. Normal will simply show the results of the search; whereas, verbose will show you more information about the algorithm and the process.</p>");
	    web.println("<p>&nbsp;</p>");

	    //prints the form for searching
	    printForm();

	} else {
	    	        
	    web.println("<h1>Multisearch: Searching...</h1>");
	    
	    //get the string and turn it into a query
	    String query = queryString.toLowerCase();
	 
	    //create the Multisearch Component
	    Multisearch search = new Multisearch(query);
	    search.setPrint(web);
	    search.setInput(inputString);
	    search.setLimit(java.lang.Integer.parseInt(limitString));
	    search.setMergeClass(algorithmString);
	    search.setOutputType(outputString);
	  
	    //ensure there is no restriction algorithm
	    if(!restrictionString.equals("None")) {

		search.setRestriction(restrictionString);

	    }

	    String[] msArgs = search.getArgs();
	    search.run(msArgs);

	    //if they wanted verbose output, now is the time to print it
	    if(viewValue.equals("true")) {

		verbose();

	    }

	    String outputFile = search.getOutputFile();

	    printResults(outputFile, outputString);

	    if(!restrictionString.equals("None")) {

		File cleanFile = new File(Multisearch.getInput());
		Multisearch.cleanInput(cleanFile);
	    }

	    //ensure cleanup
	    search = null;
	    queryString = null;
	    restrictionString = null;
	    viewValue = null;
	    algorithmString = null;
	    inputString = null;
	    outputString = null;
	    
	}

	printBottom();

    }

    /**
     * Overwrites HttpServlet's doPost.
     * 
     * @param HttpServletRequest
     * @param HttpServletResponse
     * @throws ServletException      when there are issues with connection/server  
     * @throws IOException           when there are issuses with user input  
     */
    public void doPost(HttpServletRequest request, HttpServletResponse response) 
	throws ServletException, IOException {

	doGet(request, response);

    }

    /**
     * Initalizes the servlet information, including loading the error codes
     */
    public void init() {


    }

    /**
     * Print the results from the given output file(s).
     *
     */
    public void printResults(String outputFile, String type) {

	try {

	    if(type.contains("ResultSetOutputFormat")) {

		web.println("<table style=\"border-style: solid; border-color: #000000\">");
		web.println("<tr><td class=\"tr\">Rank</td><td class=\"tr\">Title</td><td class=\"tr\">Score</td><td class=\"tr\">Server Name</td><td class=\"tr\">Old Rank</td><td class=\"tr\">Old Score</td></tr>");
		
		File file = new File(outputFile);
		
		if(file.isDirectory()) {
		    
		    File[] files = file.listFiles();
		    
		    for(int x = 0; x < files.length; x++) {
			
			File inFile = files[x];
			boolean firstline = true;

		    if(!inFile.isHidden()) {
			
			FileInputStream fis = new FileInputStream(inFile);
			DataInputStream in = new DataInputStream(fis);
			
			while(in.available() != 0) {
			    
			    if(firstline) {
				
				in.readLine();
				firstline = false;
				
			    }
			    
			    String input = in.readLine();
			    StringTokenizer st = new StringTokenizer(input);

			    String rank = st.nextToken();
			    String score = st.nextToken();
			    String serverName = st.nextToken();
			    String oldscore = st.nextToken();
			    String oldrank = st.nextToken();
			    String filename = st.nextToken();
			    String title = filename;

			    if(filename.contains("/")) {

				StringTokenizer stn = new StringTokenizer(filename, "/");
	
				while(stn.hasMoreTokens()) {

				    title = stn.nextToken();

				}

			    }

			    web.println("<tr>");
			    web.println("<td class=\"results\">" + rank + "</td>");
			    web.println("<td class=\"results\">" + title + "</td>");
			    web.println("<td class=\"results\">" + score + "</td>");
			    web.println("<td class=\"results\">" + serverName + "</td>");
			    web.println("<td class=\"results\">" + oldrank + "</td>");
			    web.println("<td class=\"results\">" + oldscore + "</td></tr>");

			}//end while-tokens
			
			fis.close();
			in.close();
			
		    }
		    
		    files[x].delete();
		    
		    } // end for-each-file
		    
		}//end-if-file-directory
		
		web.println("</table>");
		
		file.delete();

	    } else if(type.contains("TREC")) {

		web.println("<div style=\"border-style: solid; border-color: #000000; margin-left: 10%; margin-right: 10%\">");
		
		File file = new File(outputFile);
		
		if(file.isDirectory()) {
		    
		    File[] files = file.listFiles();
		    
		    for(int x = 0; x < files.length; x++) {
			
			File inFile = files[x];
			boolean firstline = true;

			if(!inFile.isHidden()) {
			
			    FileInputStream fis = new FileInputStream(inFile);
			    DataInputStream in = new DataInputStream(fis);
			    
			    while(in.available() != 0) {
				
				if(firstline) {
				    
				    in.readLine();
				    firstline = false;
				    
				}
				
				String input = in.readLine();
				web.println(input + "<br/>");
				
			    }//end while-tokens
			    
			    fis.close();
			    in.close();
			    
			}
			
			files[x].delete();
		    
		    } // end for-each-file
		    
		}//end-if-file-directory

		file.delete();
		web.println("</div>");
	    }

	} catch(Exception e) {

	    web.println("<p>There was an I/O error associated with the results.</p>");

	}
    }

    /**
     * Returns a descriptive String about Multisearch as a program.
     * 
     * @return String      describing program
     */
    public String getServletInfo() {

	return("The Arctic Region Supercomputing Center (ARSC) presents Multisearch, an engine to search for information via the grid. ARSC is using this program as research for GIR.");

    }

    /* (non-javadoc)
     * When a user calls for a more diverse view, ex--a demo, then this
     * should be called to print things out.
     */
    private void verbose() {

	web.println("<p><b>Data for this Search</b>:<br />");
	web.println("<b>Query</b>: " + queryString + "<br />");
	web.println("<b>Merge Algorithm</b>: " + algorithmString + "<br />");
	web.println("<b>Restriction Algorithm</b>: " + restrictionString + "</br>");
	web.println("<b>Output Class</b>: " + outputString + "</br/");
	web.println("<b>Input Files</b>: " + inputString + "</p>");

    }

    /* (non-javadoc)
     * Prints the XTML Form for the query to make the cod easier to read
     */
    private void printForm() {

	//this first line gives the action, which should have the same name as the
	//servlet--so if you change that in tomcat, you must change this line, too
	web.println("<form action=\"Multisearch\" method=\"POST\">");
	web.println("<p><b>Query</b>: <input type=\"text\" size=\"20\" name=\"sendQuery\"> </p>");

	//options for merge algorithm
	web.println("<p><b>Merge Algorithm</b>: <select size=\"1\" name=\"Algorithm\">");
	web.println("<option value=\"edu.arsc.multisearch.merge.LeapOfFaithMergeSet\">Leap of Faith</option>");
	web.println("<option value=\"edu.arsc.multisearch.merge.RankShuffleMergeSet\">Rank Shuffle</option>");
	web.println("<option value=\"edu.arsc.multisearch.merge.NaiveMergeSet\">Naive Merge</option>");
	web.println("</select> &nbsp; ");


	//options for Restriction algorithm
	web.println("<b>Restriction Algorithm</b>: <select size=\"1\" name=\"Restriction\">");
	web.println("<option value=\"None\">None</option>");
	web.println("<option value=\"edu.arsc.multisearch.select.RandomSelect\">Random</option>");
      	web.println("</select> &nbsp;");

	//number of restrictions
	web.println("<b>Limit</b>: <select size=\"1\" name=\"Limit\">");
	web.println("<option value=\"10\">10</option>");
	web.println("<option value=\"25\">25</option>");
	web.println("<option value=\"50\">50</option>");
	web.println("<option value=\"10\">100</option>");
      	web.println("</select> </p>");

	//options for verbose output
	web.println("<p><b>Verbose?</b> <select size=\"1\" name=\"Verbose\">");
	web.println("<option value=\"false\">No</option>");
	web.println("<option value=\"true\">Yes</option>");
	web.println("</select> &nbsp;");

	//input files
	web.println("<b>Input Files</b>: <select size=\"1\" name=\"InputFiles\">");
	web.println("<option value=\"sampleinput\">Sample</option>");
	web.println("<option value=\"allinput\">All</option>");
	web.println("</select> &nbsp;");

	//output files
	web.println("<b>OutputFormat</b>: <select size=\"1\" name=\"OutputFormat\">");
	web.println("<option value=\"edu.arsc.multisearch.ResultSetOutputFormat\">Normal</option>");
	web.println("<option value=\"edu.arsc.multisearch.TRECOutputFormat\">TREC</option>");
	web.println("</select> </p>");

	//end of form tags
	web.println("<p><input type=\"submit\" value=\"Search\"/>");
	web.println("<input type=\"reset\" value=\"Reset\"/></p>");
	web.println("</form>");

    }


    /* (non-javadoc)
     * Prints the top portion of XHTML for this page. This
     * helps make the code less complex.
     */
    private void printTop() {

	web.println("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0  Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">");

	//header stuff--important for titles and meta
	web.println("<html>");
	web.println("<head><title>Arctic Region Supercomputing Center presents Multisearch</title>");

	//meta content
	web.println("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"/>");
	web.println("<meta name=\"authors\" content=\"Kylie McCormick, Emily Teller, Chris Fallen, and mentor Greg Newby\"/>");

	//Style sheet information, making it pretty
	web.println("<link href=\"style.css\" rel=\"stylesheet\" type=\"text/css\"/>");
	web.println("<link rel=\"icon\" href=\"favicon.ico\" type=\"image/x-icon\"/>");
	web.println("</head>");

	//beginning of the visual page
	web.println("<body>");

	//some important design elements, please do not remove
	web.println("<img src=\"images_logoCircle.gif\" width=\"204\" height=\"180\" alt=\"GIR Logo\" class=\"girowly\"/>");
	web.println("<p><img src=\"images_arscheader.jpg\" width=\"448\" height=\"106\" alt=\"Arctic Region Supercomputing Center\"/></p>");

	//nav bar
	web.println("<table><tr><td class=\"nav\">");
	web.println("<p><a href=\"index.html\" class=\"nav\">Home</a></p>");

	//beginning of text location
	web.println("</td><td class=\"main\">");

    }

    /* (non-javadoc)
     * Prints the closing XHTML for the servlet; makes the code
     * less messy.
     */
    private void printBottom() {

	web.println("<p>&nbsp;</p>");
	web.println("</td></tr>");
	web.println("<tr><td class=\"bottom\" colspan=\"2\">");

	//giving credit
	web.println("<a href=\"http://www.arsc.edu/\"><img src=\"images_arsc.jpg\" width=\"120\" height=\"77\" alt=\"ARSC\" class=\"arsc\"/></a>");
	web.println("<a href=\"http://www.uaf.edu/\"><img src=\"images_uaf.png\" width=\"60\" height=\"46\" alt=\"UAF\" class=\"uaf\"/></a>");
	web.println("<p class=\"sub\">Arctic Region Supercomputing Center <br /> PO Box 756020, Fairbanks, AK 99775</p>");
	web.println("<p class=\"sub\">&copy; Arctic Region Supercomputing Center 2006-2007. This page was last updated on 8 August 2008.</p>");

	//close off all the tags
	web.println("</td></tr></table>");
	web.println("</body>");
	web.println("</html>");
    }

}
