/**
 * Client is a base class that can be extended for clients
 * to be added to Multisearch.
 *<p>
 * Late Updated: 21 July 2008
 *<p>
 * @author Kylie McCormick
 * @version 3.0
 */
package edu.arsc.multisearch.client;

import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import javax.xml.namespace.QName;
import org.apache.axis.encoding.ser.BeanSerializerFactory;
import org.apache.axis.encoding.ser.BeanDeserializerFactory;
import edu.arsc.multisearch.*;

public class LuceneAxisClient extends Client {

    /**
     * Call the service that uses this client.
     *
     * @param String query
     */
    public void call(String query) {

	try {

	    //Seralize the Objects
	    QName qName = new QName("urn:LuceneSearcher", "ResultSetWritable");
	    QName qNameDoc = new QName("urn:LuceneSearcher", "DocumentWritable");
	    BeanSerializerFactory bsfRS = new BeanSerializerFactory(edu.arsc.multisearch.backend.lucene.webservice.ResultSetWritable.class, qName);
	    BeanDeserializerFactory bdfRS = new BeanDeserializerFactory(edu.arsc.multisearch.backend.lucene.webservice.ResultSetWritable.class, qName);
	    BeanSerializerFactory bsfDoc = new BeanSerializerFactory(edu.arsc.multisearch.backend.lucene.webservice.DocumentWritable.class, qNameDoc);
	    BeanDeserializerFactory bdfDoc = new BeanDeserializerFactory(edu.arsc.multisearch.backend.lucene.webservice.DocumentWritable.class, qNameDoc);

	    //create the Axis Service calling point
	    Service service = new Service();
	    Call call = (Call)service.createCall();
	    call.setTargetEndpointAddress(new java.net.URL(URL));
	    call.registerTypeMapping(edu.arsc.multisearch.backend.lucene.webservice.ResultSetWritable.class, qName, bsfRS, bdfRS);
	    call.registerTypeMapping(edu.arsc.multisearch.backend.lucene.webservice.DocumentWritable.class, qNameDoc, bsfDoc, bdfDoc);

	    //Other factors--return type
	    call.setOperationName(new QName("urn:LuceneSearcher", "search"));
	    call.setReturnClass(edu.arsc.multisearch.backend.lucene.webservice.ResultSetWritable.class);
	    call.setReturnType(qName);

	    //SOAP Action information
	    call.setUseSOAPAction(true);
	    call.setEncodingStyle(org.apache.axis.Constants.URI_SOAP11_ENC);
	    call.setSOAPVersion(org.apache.axis.soap.SOAPConstants.SOAP11_CONSTANTS);

	    //add in/out parameters
	    call.addParameter("in0", org.apache.axis.encoding.XMLType.XSD_STRING, javax.xml.rpc.ParameterMode.IN);
	    call.addParameter("ResultSetWritable", qName, edu.arsc.multisearch.backend.lucene.webservice.ResultSetWritable.class, javax.xml.rpc.ParameterMode.OUT);
	    call.addParameter("DocumentWritable", qNameDoc, edu.arsc.multisearch.backend.lucene.webservice.DocumentWritable.class, javax.xml.rpc.ParameterMode.OUT);
	    
	    //generate input for service--the query
	    String[] param = new String[1];
	    param[0] = query;

	    //make the call
	    edu.arsc.multisearch.backend.lucene.webservice.ResultSetWritable result = (edu.arsc.multisearch.backend.lucene.webservice.ResultSetWritable)call.invoke(param);	    


	    if(result != null) {

		//rsw = new ResultSetWritable();		
		//rsw.setName(result.getName());
		//rsw.setInfo(result.getInfo());

		edu.arsc.multisearch.backend.lucene.webservice.DocumentWritable[] docs = result.getResults();

		if(docs == null) {
		    
		    rsw.setResults(null);
		    throw new java.lang.NullPointerException("Docs null still from " +  result.getName());

		} else {



		    DocumentWritable[] doc = new DocumentWritable[docs.length];

		    for(int x = 0; x < docs.length; x++) {

			doc[x] = new DocumentWritable(x, docs[x].getTitle(),
					      docs[x].getScore(),
					      docs[x].getFilename(),
					      docs[x].getName());

		    }

		    //rsw.setResults(doc);
		    rsw = new ResultSetWritable(doc, result.getName(), result.getInfo());		

		}

	      

		
	    } else {

		throw new java.lang.NullPointerException("Result Set Null.");

	    }


	} catch(Exception e) {

	    System.err.println("@AxisClient: " + e.toString());
	    e.printStackTrace();
	    
	}


    }




}
