/**
 * Extention of the OGSA-DAI Activity, which allows for
 * new creators of Multisearch to add new Client Toolkit
 * Activities for new resources. This allows for flexible
 * perform documents.
 * <p>
 * This requires various things that Multisearch uses to
 * search different types of data. New Activities should
 * extend this.
 * <p>
 * DATE CREATED: 17 July 2007
 * LAST UPDATED: 25 July 2007
 *
 * @author OGSA-DAI, Kylie McCormick
 */
package edu.arsc.multisearch.client;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import uk.org.ogsadai.client.toolkit.ResourceID;
import uk.org.ogsadai.client.toolkit.activity.*;

public class SearchToolkit extends Activity
{

    /** Index of search data output */
    protected static final int DATA_OUTPUT_INDEX = 0;

    /** Index of search status output */
    protected static final int STATUS_OUTPUT_INDEX = 1;

    /** 
     * A list of the activity's input parameters 
     * (<code>ActivityInputParameter</code>)
     */
    protected List mInputParams;

    /** 
     * A list of the activity's output parameters 
     * (<code>ActivityOuyputParameter</code>)
     */
    protected List mOutputParams;

    /** The query object associated with this search. */
    protected String query;

    /** The minimum number of results from this activity.*/
    protected int min;

    /** The maximum number of results from this activity. */
    protected int max;

    /** The index associated with this search. */
    protected String index;

    /**
     * Constructs an activity with no inputs or outputs.
     */
    public SearchToolkit()
    {
	super();
    }
    
    /**
     * This allows forClass().getInstance() structure
     */
    public SearchToolkit getInstance() {

	return(new SearchToolkit());
    }
    
    /* (non-Javadoc)
     * @see uk.org.ogsadai.client.toolkit.RequestComponent#generateXML()
     */
    protected String generateXML() {
	
	String newString = "";
	
	return(newString);

    }
    
    /**
     * Set the Minimum result requirement.
     */
    public void setMin(int min) {
	
	this.min = min;

    }

    /** 
     * Set the Maximum result limit.
     */
    public void setMax(int max) {

	this.max = max;
    }

    /**
     * Set the query, string.
     */
    public void setQuery(String query) {

	this.query = query;

    }

    /** 
     * Set the index of this search, if there is one (might be NULL)
     */
    public void setIndex(String index) {

	this.index = index;

    }

    /**
     * Gets the activity's data output.
     * 
     * @return the activity output.
     */    
    public ActivityOutput getDataOutput()
    {
        ActivityOutput queryResultOutput = getOutputs()[DATA_OUTPUT_INDEX];
        return queryResultOutput;
    }

    /**
     * Gets the activity's search status output.
     * 
     * @return the search status.
     */    
    public ActivityOutput getSearchStatus()
    {
	
        ActivityOutput queryResultOutput = getOutputs()[STATUS_OUTPUT_INDEX];
        return queryResultOutput;

    }


}
