 /**
  * Rank Shuffle does no reweighting of the merge sets, but simply uses
  * the ranks as the document scores.
  * <p>
  * Last Updated: 31 July 2008
  *
  * @author Kylie McCormick
  * @version 3.0
  */
package edu.arsc.multisearch.merge;

import edu.arsc.multisearch.*;
import java.util.LinkedList;

public class RankShuffleMergeSet extends FinalSet {


    /**
     * Creates a new NaiveMergeSet to allow merging of Documents
     */
    public RankShuffleMergeSet() {

	results = null;
	rankedList = new OrderedList();

    }

    /**
     * Allows for reflection.
     *
     * @return LeapOfFaithMergeSet
     */
    public RankShuffleMergeSet getInstance() {
	
	return(new RankShuffleMergeSet());
    }

    /**
     * A merge object based on the Naive Merge Algorithm.
     * 
     * @param DocumentWritable[] hits        the documents to be folded into the finalset
     */
    public void merge(DocumentWritable[] hits) {

	if(hits != null) {

	    for(int i = 0; i < hits.length; i++) {

		DocumentWritable doc = hits[i];
		doc.setScore(doc.getOldRank()*-1); //negative to ensure ranking is correct
		rankedList.addRanked(doc);

	    }

	    rankedList.setRanks();

	        
	}

    }	

    /**
     * A function that returns the class name of this algorithm.
     * 
     * @return String className
     */
    public String getMergeAlgorithm() {

	return("edu.arsc.multisearch.merge.RankShuffleMergeSet");

    }

}
