/**
 * A class that defines the merge possibility for Multisearch.
 *
 * @author Kylie McCormick
 * @project Multisearch
 * @date 14 July 2008
 */
package edu.arsc.multisearch;

import java.util.Iterator;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.conf.*;
import org.apache.hadoop.io.*;
import org.apache.hadoop.mapred.*;
import org.apache.hadoop.util.*;
import edu.arsc.multisearch.merge.*;

public class MergeReduce extends MapReduceBase implements Reducer<Text, ResultSetWritable, Text, ResultSetWritable> {

    /** Counters via Hadoop.*/
    static enum Counters {INPUT_SERVICES, OUTPUT_RESULTS};

    /**
     * The actual reduce function.
     *
     * @param Text text
     * @param Iterator<ServiceWritable> iterator
     * @param OutputCollector<Text, ResultSetWritable> outputCol
     * @param Reporter reporter
     */
    public void reduce(Text text, Iterator<ResultSetWritable> iterator, OutputCollector<Text, ResultSetWritable> outputCol, Reporter reporter) {

	//create the appropriate kind of final set (naive merge for now)
	NaiveMergeSet nms = new NaiveMergeSet();

	//iterate through the ServiceWritable, taking the output and merging it
	while(iterator.hasNext()) {

	    reporter.incrCounter(Counters.OUTPUT_RESULTS, 1);

	    //grab the output result set
	    ResultSetWritable rsw = iterator.next();
	    ResultSet rs = rsw.get();
	    DocumentWritable[] docs = rs.getResults();

	    reporter.setStatus("Current input from iterator: " + rs.getName() + " with key " + text.toString());

	    if(docs != null) {

		//merge them together
		nms.merge(docs);
	    
	    } 

	} //end-while-has-iterator


	ResultSetWritable rsw = new ResultSetWritable(nms.getResults(), "FinalSet", "FinalSet for Multisearch");
	Text newKey = new Text(Multisearch.getQuery());
	
	try {

	    //collect the merged function
	    outputCol.collect(newKey, rsw);
	    
	} catch(java.io.IOException ioe) {
	    
	    reporter.setStatus(ioe.toString());
	    System.out.println("@MergeReduce: "  + ioe.toString());
	    
	} 
	
	 
    }

}





