/**
 * The object that creates a distributed search with Apache Hadoop and runs
 * it. Also contains the main function needed to run Multisearch from the
 * command line.
 * <p>
 *
 * @author Kylie McCormick
 * @project Multisearch
 * @date 26 June 2008
 * @update 21 July 2008
 */
package edu.arsc.multisearch;

import org.apache.hadoop.mapred.jobcontrol.*;
import java.io.File;
import java.io.IOException;
import java.io.DataInputStream;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.util.StringTokenizer;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.conf.*;
import org.apache.hadoop.io.*;
import org.apache.hadoop.mapred.*;
import org.apache.hadoop.util.*;
import org.apache.hadoop.mapred.lib.IdentityReducer;

public class Multisearch extends Configured implements Tool {

    /** The query for the multisearch. */
    private static String multisearchSuperQuery;

    /** The file for the multisearch, assuming a batch. */
    private File file;

    /** Counters via Hadoop.*/
    static enum Counters {INPUT_SERVICES, OUTPUT_RESULTS};

    //some other stuff
    private String inputFile;

    /**
     * Runs a Multisearch and prints the results.
     */
    public static void main(String args[]) {

	if(args.length == 0 || args.length == 1) {

	    System.out.println("Please enter either a -q query or -f file with queries.");
	    System.out.println("Files with queries should be formatted num:query.");


	} else if(args[0].equals("-q")) {

	    String query = "";

	    for(int x = 1; x < args.length; x++) {
		
		query += " " + args[x];

	    }

	    query = query.trim();
	    query = query.toLowerCase();
	 
	    Multisearch multisearch = new Multisearch(query);


	} else if(args[0].equals("-f")) {

	    String filename = args[1];
	    File file = new File(filename);
	    Multisearch multisearch = new Multisearch(file);


	} //end if-args

	
    }

    /**
     * Create a Multisearch object with only one query.
     *
     * @param String query
     */
    public Multisearch(String query) {

	this.multisearchSuperQuery = query;
	System.out.println("The query " + query + " is being run.");
	String[] q = new String[1];
	q[0] = query;
	int value = run(q);

    }

    /**
     * Create a Multisearch object with a file full of queries.
     *
     * @param File file
     */
    public Multisearch(File file) {

	try {

	    //Open the need File Inputs
	    FileInputStream fis = new FileInputStream(file);
	    BufferedInputStream bis = new BufferedInputStream(fis);
	    DataInputStream dis = new DataInputStream(bis);
	    BufferedReader in = new BufferedReader(new InputStreamReader(dis));

	    //Extract the queries and run this
	    while(dis.available() != 0) {

		String line = in.readLine();
		StringTokenizer st = new StringTokenizer(line, ":");
		String qNum = st.nextToken();
		String query = st.nextToken();
	    
		query = query.trim();
		query = query.toLowerCase();

		String[] q = new String[2];
		q[0] = query;
		q[1] = qNum;
		int value = run(q);

	    }

	    //close the file inputs
	    in.close();
	    dis.close();
	    bis.close();
	    fis.close();

	} catch(java.io.FileNotFoundException fnfe) {

	    System.out.println(fnfe.toString());

	} catch(java.io.IOException ioe) {

	    System.out.println(ioe.toString());

	}


    }


    /**
     * Run function
     */
    public int run(String args[]) {

	//JobConf conf = new JobConf(getConf(), Multisearch.class);
	JobConf conf = new JobConf(new Configuration(), Multisearch.class);
	conf.setJobName("multisearch");
	conf.setSessionId("id349349sd");
	conf.setJar("/home/mccormic/merge/programs/MultisearchTemp.jar");

	//Paths
	ServiceInputFormat.setInputPaths(conf, new Path("/home/mccormic/merge/programs/inputfiles/"));
	ResultSetOutputFormat.setOutputPath(conf, new Path("/home/mccormic/merge/programs/hadoopOutput"));

	//Service and Merge classes
	conf.setMapperClass(edu.arsc.multisearch.ServiceMap.class);
	conf.setCombinerClass(edu.arsc.multisearch.MergeReduce.class);
	conf.setReducerClass(edu.arsc.multisearch.MergeReduce.class);

	//Mapper outputs
	conf.setMapOutputKeyClass(Text.class);
	conf.setMapOutputValueClass(edu.arsc.multisearch.ResultSetWritable.class);

	//Merge outputs
	conf.setOutputKeyClass(Text.class);
	conf.setOutputValueClass(edu.arsc.multisearch.ResultSetWritable.class);

	//things that might not be needed
	//conf.setNumReduceTasks(1);
	conf.setCompressMapOutput(false);

	//input/output formats
	conf.setOutputFormat(edu.arsc.multisearch.ResultSetOutputFormat.class);
	//conf.setOutputFormat(edu.arsc.multisearch.TRECOutputFormat.class);
	conf.setInputFormat(edu.arsc.multisearch.ServiceInputFormat.class);

	try {

	    JobClient.runJob(conf);
	    
	} catch(java.io.IOException ioe) {

	    System.err.println("@Multisearch.run(): " + ioe.toString());

	} catch(java.lang.Exception e) {

	    System.err.println("@Multisearch.run(): " + e.toString());
	    e.printStackTrace();

	}

	System.out.println("complete.");

	return(1);

    }


    /**
     * Return the query associated with this Multisearch object.
     *
     * @return String
     */
    public static String getQuery() {

	return(multisearchSuperQuery);

    }

}
