/**
 * An object that represents a multisearch result set that matches
 * to other searches in Multisearch. Can be serialized with NetBeans.
 *
 * @author Kylie McCormick
 * @date 22 July 2008
 * @project multisearch
 */
package edu.arsc.multisearch;

public class ResultSet {

    /** Array of results, assumed in order. */
    private DocumentWritable[] results;

    /** Name of the server/backend from which these results came.*/
    private String name;

    /** Any other information the server/backend wants to be known.*/
    private String info;

    /** OrderedList for new documents in FinalSet.*/
    private OrderedList ranked;

    /**
     * needed for Hadoop's Reflection
     */
    public ResultSet() {

	this.results = null;
	this.name = "Empty Instance";
	this.info = "This is an empty instance of a result set.";

    }

    /**
     * Create a new resultset.
     *
     * @param Document[] results       the ordered documents
     * @param String name              the server name
     * @param String info              any additional information
     */
    public ResultSet(DocumentWritable[] results, String name, String info) {

	this.results = results;
	this.name = name;
	this.info = info;
	this.ranked = new OrderedList();

    }

    /**
     * @param DocumentWritable[] setResults
     */
    public void addResults(DocumentWritable[] newSet) {

	//First, add them to the OrderedList
	for(int x = 0; x < newSet.length; x++) {
	    
	    ranked.addRanked(newSet[x]);
	    
	}

	if(results == null) {

	    //our first set!
	    results = newSet;

	} else {

	    results = null;
	    results = new DocumentWritable[ranked.size()];
	    
	    for(int i = 0; i < ranked.size(); i++) {

		results[i] = (DocumentWritable)ranked.get(i);

	    }

	}


    }


    /**
     * @return DocumentWritable[] results
     */
    public DocumentWritable[] getResults() {

	return(results);

    }

    /**
     * @return String name
     */
    public String getName() {

	return(name);

    }

    /**
     * @return String info
     */
    public String getInfo() {

	return(info);

    }

    /** 
     * @param String name
     */
    public void setName(String name) {

	this.name = name;

    }

    /**
     * @param String info
     */
    public void setInfo(String info) {

	this.info = info;

    }

    /**
     * @param DocumentWritable[] results
     */
    public void setResults(DocumentWritable[] results) {

	this.results = results;

    }

}
