/**
 * An object that represents a multisearch result set that matches
 * to other searches in Multisearch. Can be serialized with NetBeans.
 *
 * @author Kylie McCormick
 * @date 22 July 2008
 * @project multisearch
 */
package edu.arsc.multisearch;

import java.io.DataInput;
import java.io.DataOutput;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.conf.*;
import org.apache.hadoop.io.*;
import org.apache.hadoop.mapred.*;
import org.apache.hadoop.util.*;
import java.util.StringTokenizer;
import java.util.LinkedList;

public class ResultSetWritable implements Writable {

    /** The object being serialized **/
    private ResultSet rset;

    /**
     * @param DataInput
     */
    public void readFields(DataInput input) {

	try {


	    String line = input.readLine();
	    StringTokenizer st = new StringTokenizer(line, "=");

	    /**
	       Format, as in write() is the following:
	       Name:
	       Info:
	       Documents: (as many as there are)

		    lineInput += "=" + (i+1) + "=" + docs[i].getTitle() + "=" + 
			docs[i].getScore() + "=" + docs[i].getOldRank() + "=" + 
			docs[i].getOldScore()+ "=" + docs[i].getFilename();		

			=rank=title=score=oldrank=oldscore=filename
	     **/

	    //String nothing = st.nextToken();
	    String name = st.nextToken();
	    String info = st.nextToken();
	    LinkedList ll = new LinkedList();

	    //Document Collection
	    while(st.hasMoreTokens()){

		boolean complete = false;

		while(!complete && st.hasMoreTokens()) {
		    
		    String rank = st.nextToken();
		    String title = st.nextToken();
		    String score = st.nextToken();
		    String oldrank = st.nextToken();
		    String oldscore = st.nextToken();
		    String filename = st.nextToken();
		    String nameDoc = st.nextToken();

		    int rankI = Integer.parseInt(rank);
		    int oldrankI = Integer.parseInt(oldrank);
		    double scoreD = Double.parseDouble(score);
		    double oldscoreD = Double.parseDouble(oldscore);

		    DocumentWritable dw = new DocumentWritable(rankI, oldrankI, title, scoreD, oldscoreD, filename, nameDoc);
		    ll.add(dw);
		    
		    complete = true;

		}
		

	    }

	    DocumentWritable[] docs = new DocumentWritable[ll.size()];

	    for(int x = 0; x < ll.size(); x++) {

		docs[x] = (DocumentWritable)ll.get(x);

	    }

	    rset.setInfo(info);
	    rset.setName(name);
	    rset.setResults(docs);


	} catch(java.io.IOException ioe) {


	}

    }

    /**
     * @param DataOutput
     */
    public void write(DataOutput output) {

	try {

	    String lineInput = rset.getName() + "=" + rset.getInfo();
	    DocumentWritable[] docs = rset.getResults();

	    if(docs != null) {

		for(int i = 0; i < docs.length; i++) {
		    
		    String title = docs[i].getTitle();
		    double score = docs[i].getScore();
		    int oldrank = docs[i].getOldRank();
		    double oldscore = docs[i].getOldScore();
		    String filename = docs[i].getFilename();
		    String name = docs[i].getName();

		    if(title.equals("")) {
			
			title = "untitled";
		    }

		    if(filename.equals("")) {

			filename = "unknown";

		    }

		    if(name.equals("")) {

			name = "unknown";

		    }


		    
		    lineInput += "=" + (i+1) + "=" + title + "=" + score + "=" + oldrank + "=" + oldscore + "=" + filename + "=" + name;

		}

		
	    }

	    lineInput+= "\n";
	    //byte[] bytes = lineInput.getBytes("UTF-8");
	    //Text writer = new Text(lineInput);
	    output.writeBytes(lineInput);
	    //output.writeUTF(lineInput.toUTF());
	    //output.write(bytes, 0, bytes.length);

	} catch(java.io.IOException ioe) {

	    

	}

    }

    /**
     * needed for Hadoop's Reflection
     */
    public ResultSetWritable() {
	
	rset = new ResultSet();
	
    }

    /**
     * Create a new resultset.
     *
     * @param Document[] results       the ordered documents
     * @param String name              the server name
     * @param String info              any additional information
     */
    public ResultSetWritable(DocumentWritable[] results, String name, String info) {

	rset = new ResultSet(results, name, info);

    }

    /**
     * @param DocumentWritable[] setResults
     */
    public void addResults(DocumentWritable[] newSet) {

	rset.addResults(newSet);

    }


    /**
     * @return DocumentWritable[] results
     */
    public DocumentWritable[] getResults() {

	return(rset.getResults());

    }

    /**
     * @return String name
     */
    public String getName() {

	return(rset.getName());

    }

    /**
     * @return String info
     */
    public String getInfo() {

	return(rset.getInfo());

    }

    /** 
     * @param String name
     */
    public void setName(String name) {

	rset.setName(name);

    }

    /**
     * @param String info
     */
    public void setInfo(String info) {

	rset.setInfo(info);

    }

    /**
     * @param DocumentWritable[] results
     */
    public void setResults(DocumentWritable[] results) {

	rset.setResults(results);

    }

    /**
     * @param ResultSet
     */
    public void set(ResultSet rs) {

	rset = rs;

    }

    /**
     * @return ResultSet
     */
    public ResultSet get() {

	return(rset);

    }

}
