/**
 * ServiceMap is extended from Hadoop.
 *
 */
package edu.arsc.multisearch;

import org.apache.hadoop.fs.Path;
import org.apache.hadoop.conf.*;
import org.apache.hadoop.io.*;
import org.apache.hadoop.mapred.*;
import org.apache.hadoop.util.*;

public class ServiceMap extends MapReduceBase implements Mapper<Text, ServiceWritable, Text, ResultSetWritable> {

    /** Counters via Hadoop.*/
    static enum Counters {INPUT_SERVICES, OUTPUT_RESULTS};

    /** 
     * The map function.
     *
     * @param ServiceWritable serviceW
     * @param Text text
     * @param OutputCollector<Text,ResultSetWritable>  outputCol
     * @param Reporter reporter
     */
    public void map(Text text, ServiceWritable serviceW, OutputCollector<Text, ResultSetWritable> outputCol, Reporter reporter) {

	try {

	    //Figure out which client we want
	    String clientClass = serviceW.getClientClass();
	    Class clientMaker = Class.forName(clientClass);

	    //create the client
	    Client client = (Client)clientMaker.newInstance();
	    reporter.setStatus("Starting client " + serviceW.getName() + " from key " + text.toString());

	    //Set the URL
	    client.setURL(serviceW.getURL());

	    //Take the query
	    String query = text.toString();

	    //Run the call
	    client.call(query);

	    //get the resultset
	    ResultSetWritable newValue = client.getResultSet();

	    //new text
	    Text newKey = new Text(Multisearch.getQuery());

	    //output collector
	    outputCol.collect(newKey, newValue);

	    //status
	    reporter.setStatus("Collecting " + newValue.getName() + " from key " + newKey.toString());
	    reporter.incrCounter(Counters.INPUT_SERVICES, 1);


	} catch(java.lang.InstantiationException ie) {

	    reporter.setStatus(new String("Cannot instantiate given client class : " + ie.toString()));
	    System.err.println("@ServiceMap: Cannot instantiate given client class. " + ie.toString());

	} catch(java.lang.ClassNotFoundException cnfe) {

	    reporter.setStatus(new String("Client Class specified does not exist: " + cnfe.toString()));
	    System.err.println("@ServiceMap: Client class does not exist. " + cnfe.toString());

	} catch(java.lang.IllegalAccessException iae) {

	    reporter.setStatus(new String("IllegalAccess: Class not accessible: " + iae.toString()));
	    System.err.println("@ServiceMap: Class not accessible " + iae.toString());
	    
	} catch(java.io.IOException ioe) {

	    reporter.setStatus(new String("IO Error from output collector: " + ioe.toString()));
	    System.err.println("@ServiceMap: IO Error " + ioe.toString());

	} catch(java.lang.NullPointerException npe) {

	    reporter.setStatus(npe.toString());
	    System.err.println("@ServiceMap: Null error: " + npe.toString());

	} catch(java.lang.Exception e) {

	    reporter.setStatus(new String("Error from Client: " + e.toString()));
	    System.err.println("@ServiceMap: Client has responded with error. " + e.toString());
	    e.printStackTrace();

	}

    }
    




}
