/**
 * Key/Value pair defined for Hadoop.
 *
 * @author Kylie McCormick
 * @date 22 July 2008
 *
 */
package edu.arsc.multisearch;

import java.io.IOException;
import java.io.DataInputStream;
import java.io.InputStream;
import java.io.DataOutputStream;
import java.io.OutputStream;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.conf.*;
import org.apache.hadoop.io.*;
import org.apache.hadoop.mapred.*;
import org.apache.hadoop.util.*;

public class ServiceWritable implements Writable {

    /** URI of service */
    private String URL;

    /** Client class needed for service. */
    private String clientClass;

    /** ResultSet including the documents for service.*/
    private ResultSetWritable rsw;

    /** The special name (specific to TREC or otherwise) for this service.*/
    private String name;

    /**
     * This must be overwritten for Hadoop.
     *
     * @param java.io.DataInput
     */     
    public void readFields(java.io.DataInput in) throws IOException {

    }

   /**
    * This must be overwritten for Hadoop.
    *
    * @param java.io.DataOutput
    */   
    public void write(java.io.DataOutput out) throws IOException {

    }

    /**
     *
     */
    public ServiceWritable() {

	name = "empty instance";
	clientClass = "";
	URL = "";

    }


    /**
     * Set the spec name for this.
     *
     * @param String name
     */
    public void setName(String name) {

	this.name = name;

    }

    /**
     * Sets the results.
     *
     * @param DocumentWritable[] documents
     */
    public void setResults(DocumentWritable[] docs) {

	rsw.setResults(docs);

    }

    /**
     * @return String name
     */
    public String getName() {

	return(name);

    }


    /**
     * Sets the URI value for the service.
     *
     * @param String URI
     */
    public void setURL(String URL) {

	this.URL = URL;

    }

    /**
     * Sets the clientClass value for the service.
     *
     * @param String clientClass
     */
    public void setClientClass(String clientClass) {

	this.clientClass = clientClass;

    }

    /**
     * Gets the clientClass value for this service.
     *
     * @return String clientClass
     */
    public String getClientClass() {

	return(clientClass);

    }


    /**
     * Gets the URI value for this service.
     *
     * @return String URI
     */
    public String getURL() {

	return(URL);

    }

    /**
     * This allows for easy access to documents wrapped in resultset.
     *
     * @return ResultSetWritable
     */
    public ResultSetWritable get() {

	return(rsw);

    }

    /**
     * Easy access!
     *
     * @param ResultSetWritable
     */ 
     public void set(ResultSetWritable rsw) {

	 this.rsw = rsw;

     }

}
