/*
 * NAIVEMERGESET
 * The "final set" of ResultSets that are merged given
 * mapping to the Logistical Regression
 *
 * Some code taken from Aaron Luptak's 2005 code
 *
 * AUTHOR: McCormick, Kylie
 * DATE CREATED: 25 July 2006
 * LAST UPDATED: 7 July 2008
 * PROJECT: GridIR with Newby and Fallen
 *
 */
package edu.arsc.multisearch.backend;

import java.io.PrintWriter;
import java.util.LinkedList;

public class NaiveMergeSet extends FinalSet {


    /*
     * CONSTRUCTOR FUNCTION
     *
     * INPUT: A PrintWriter Location
     * OUTPUT: New Object
     *
     */
    public NaiveMergeSet() {

	try {

	    totalResults = 0;
	    rankedList = new OrderedList();

	} catch(Exception e) {

	    e.printStackTrace();
	}

    }

    /*
     * A simple print function
     */
    public void print() {

	System.out.println("NAIVE MERGE SET RESULTS");
	System.out.println();

	System.out.println("Rank \t Score \t\t Server \t Old Rank \t Old Score \t\t Filename");

	for(int x = 0; x < rankedList.size(); x++) {

	    Document doc = (Document)rankedList.get(x);

	    System.out.println((x+1) + "\t" + doc.getScore() + "\t" + doc.getName() + "\t" + doc.getOldRank() + "\t" + doc.getOldScore() + "\t" + doc.getFilename());

	}
	
    }


    /* 
     * MERGE
     * This function takes in a result set, rescores each result
     * and then adds it to the final ranked list at the end.
     *
     * NOTE: This code is from Aaron Luptak's 2005 Code
     *
     * INPUT: ResultSet 
     * OUTPUT: None
     *
     */
    public void merge(ResultSet rs) {

	//Important variables
	Document[] hits = rs.getResults();

	hits = fromNeg(hits);
	//rs.setResults(hits);

	int numresults = hits.length;
	double alpha;
	double beta;
	double sumX=0;
	double sumY=0; 
	double sumXY=0;
	double sumX2=0;
	    
	try {


	    for(int i = 0; i < numresults; i++) {
		
		double x = java.lang.Math.log((double)(i+1));
		double y = 0;

		Document doc = hits[i];

		if(doc.getScore() > 0) {

		    double G = (double)(1-(doc.getScore() >= 1? .9999999 : doc.getScore())); 
		    G = (double)(doc.getScore()/G);
		    y = java.lang.Math.log(G);

		} 
		
		//sum the variables
		sumX += x;
		sumY += y;
		sumXY += (x * y);
		sumX2 += (x * x);


	    } //end for-each-doc


	    /*
	    //Calculates stats
	    //Will be used for later calculations
	    for(int i =0; i < numresults; i++) {

		double x = java.lang.Math.log((double)(i+1));
		double y;
		try {
		    Document doc = hits[i];
		    if(doc.getScore() != 0) {
			y = java.lang.Math.log((double)(doc.getScore()/(1-(doc.getScore() >= 1? .9999999 : doc.getScore()))));
		    
		    } else {
			y = 0;
		    }
		} catch(Exception e) {
		y = 0;
		    e.printStackTrace();
		}
		    
		sumX += x;
		sumY += y;
		sumXY += x * y;
		sumX2 += x * x;
	    }
	    */
	    double D = sumX2*numresults - sumX*sumX;
	    beta = (sumXY*numresults - sumX*sumY)/D;
	    alpha = (sumX2*sumY - sumXY*sumX)/D;
	    
	    //This goes through each result and, using the stats above, calculates
	    //each result's new score
	    for(int x = 0; x < numresults; x++) {
		
		try {

		    Document hit = hits[x];
		    double newscore = java.lang.Math.exp(alpha+beta*java.lang.Math.log((double)x+1))/(1+java.lang.Math.exp(alpha+beta*java.lang.Math.log((double)x+1)));

		    hit.setScore(newscore);
		    rankedList.addRanked(hit);

		    if(x == numresults-1) {
			
			rankedList.setRanks();
			totalResults = rankedList.size();
		    }


		} catch(Exception e) {
		
		    e.printStackTrace();
	
		}
		
	    }   

	    
	} catch(Exception e) {

	    e.printStackTrace();

	}

    }

    /**
     * Takes negatively ranked documents and ranks them with positive numbers
     *
     * @param Document[]
     * @return Document[]
     */
    public Document[] fromNeg(Document[] hits) {

	try {

	    if(hits.length > 2) {

		double maxScore = hits[0].getScore();
		double minScore = hits[hits.length-1].getScore();
	    


		for(int i = 0; i < hits.length; i++) {

		    double score = hits[i].getScore();
		    score = score - minScore;
		    score = score / (maxScore - minScore);
		    hits[i].setScore(score);
		}

	    }


	} catch(Exception e) {

	    e.printStackTrace();

	}

	return(hits);
    }

}
