/**
 * An object that represents a multisearch result set that matches
 * to other searches in Multisearch. Can be serialized with NetBeans.
 *
 * @author Kylie McCormick
 * @date 2 July 2008
 * @project multisearch
 */
package edu.arsc.multisearch.backend;

public class ResultSet {

    /** Array of results, assumed in order. */
    private Document[] results;

    /** Name of the server/backend from which these results came.*/
    private String name;

    /** Any other information the server/backend wants to be known.*/
    private String info;
     

    /**
     * Create a new resultset.
     *
     * @param Document[] results       the ordered documents
     * @param String name              the server name
     * @param String info              any additional information
     */
    public ResultSet(Document[] results, String name, String info) {

	this.results = results;
	this.name = name;
	this.info = info;

    }

    /**
     * Make an empty instance.
     */
    public ResultSet() {

	results = null;
	name = "";
	info = "This is an empty instance.";

    }

    /**
     * @return Document[] results
     */
    public Document[] getResults() {

	return(results);

    }

    /**
     * @return String name
     */
    public String getName() {

	return(name);

    }

    /**
     * @return String info
     */
    public String getInfo() {

	return(info);

    }

    /** 
     * @param String name
     */
    public void setName(String name) {

	this.name = name;

    }

    /**
     * @param String info
     */
    public void setInfo(String info) {

	this.info = info;

    }

    /**
     * @param Document[] results
     */
    public void setResults(Document[] results) {

	this.results = results;

    }

}
