/**
 * An object to search a Lemur-made index.
 *
 * @author Kylie McCormick
 * @project Multisearch
 * @date 25 June 2008
 *
 */
package edu.arsc.multisearch.backend.lemur;

import java.util.LinkedList;
import lemurproject.indri.*;
//import edu.arsc.multisearch.backend.*;

public class LemurIndriSearcherImpl {

    /** Base URI. **/
    private static String baseDir = "/home/mccormic/merge/indexes/lemur/indri/gx030/";

    /**
     * A main function to run from the command line
     *
     **/
    public static void main(String[] args) throws Exception {

	QueryEnvironment env = new QueryEnvironment();

	String myIndex = baseDir;

	String myQuery = "";

	for(int i = 0; i < args.length; i++) {

	    myQuery += " " + args[i];

	}

	myQuery = myQuery.trim();

	ScoredExtentResult[] results;
	String[] names;
	String[] titles;

	env.addIndex(myIndex);
	results = env.runQuery(myQuery, 1000);
	

	names = env.documentMetadata(results, "docno");
	titles = env.documentMetadata(results, "title");

	for(int i=0; i < results.length; i++) {

	    System.out.println((i+1) + " " + names[i] + " " + 
			       results[i].score + " " + 
			       results[i].begin + " " +
			       results[i].end + " " + titles[i]);

	}

    
	env.close();
	

    }

    /** 
     * new object
     */
    public LemurIndriSearcherImpl() {



    }

    /**
     * @param String query
     * @return ResultSet
     */
    public edu.arsc.multisearch.backend.lemur.indriws.ResultSetWritable search(String query) {


	LinkedList list = runLemur(query);

	edu.arsc.multisearch.backend.lemur.indriws.DocumentWritable[] resultList = new edu.arsc.multisearch.backend.lemur.indriws.DocumentWritable[list.size()];

	for(int x = 0; x < list.size(); x++) {
	    
	    resultList[x] = (edu.arsc.multisearch.backend.lemur.indriws.DocumentWritable)list.get(x);
	    
	}
	
	
	edu.arsc.multisearch.backend.lemur.indriws.ResultSetWritable set = new edu.arsc.multisearch.backend.lemur.indriws.ResultSetWritable("Lemur via Indri - No Stop words used", "LemurIndriSearcher", resultList);

	return(set);



    }

    private LinkedList runLemur(String query) {

	LinkedList list = new LinkedList();

	try {

	    QueryEnvironment env = new QueryEnvironment();
	    String myIndex = baseDir;
	    String myQuery = query;
	
	    ScoredExtentResult[] results;
	    String[] names;
	    String[] titles;

	    env.addIndex(myIndex);
	    results = env.runQuery(myQuery, 1000);
	
	    titles = env.documentMetadata(results, "title");
	    names = env.documentMetadata(results, "docno");

	    
	

	    for(int i=0; i < results.length; i++) {

		//DocumentWritable doc = new DocumentWritable(names[i], "LemurIndriSearcher", (i+1), results[i].score, (i+1), results[i].score, titles[i]);

		String title = titles[i];

		while(title.contains("=")) {

		    title = title.replace("=", "");

		}

		String eol = "\n";
		
		while(title.contains(eol)) {

		    title = title.replace(eol, "");
		}

		String oddball = "\r";
		while(title.contains(oddball)) {

		    title = title.replaceAll(oddball, "");

		}

		title = title.trim();


		edu.arsc.multisearch.backend.lemur.indriws.DocumentWritable doc = new edu.arsc.multisearch.backend.lemur.indriws.DocumentWritable(names[i], "LemurIndriSearcher", (i+1), results[i].score, (i+1), results[i].score, title);
		list.add(doc);
	

	    }

	    env.close();

	} catch(Exception e) {

	    e.printStackTrace();

	}

	return(list);

    }

}
