/**
 * An object used to search a given Lucene Index via Multisearch/Hadoop.
 *
 * @author Kylie McCormick
 * @project Multsiearch
 * @date 23 June 2008
 */
package edu.arsc.multisearch.backend.lucene;

import java.io.IOException;
import java.util.LinkedList;

import org.apache.lucene.search.Hits;
import org.apache.lucene.search.Query;
import org.apache.lucene.document.Field;
import org.apache.lucene.search.Searcher;
import org.apache.lucene.document.Document;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.analysis.standard.StandardAnalyzer;


public class LuceneSearcherImpl {

    
    /** The base URI of the files.**/
    public static String index = "/home/mccormic/merge/indexes/gov2.dsub.1165/luc";

    /**
     * Runs the search from the command line.
     *
     */
    public static void main(String args[]) {

	String query = "";
	String index = "";

	if(args.length > 0) {

	    for(int x = 0; x < args.length; x++) {

		query += " " + args[x];

	    }

	    query = query.trim();
	}

	//searchit(query);

	LuceneSearcherImpl lsi = new LuceneSearcherImpl();

	//edu.arsc.multisearch.backend.lucene.webservice.ResultSet rs = lsi.search(query);
	
	//edu.arsc.multisearch.backend.lucene.webservice.Document[] docs = rs.getResults();
	/*
	for(int x = 0; x < docs.length; x++) {

	    System.out.println((x+1) + " " + docs[x].getFilename());

	    }*/

    }


    /**
     * Runs the search given a query and a particular index
     *
     * @param String queryString             the query to be searched for
     */
    public static void searchit(String queryString) {

	try {

	    Searcher searcher = new IndexSearcher(index);;

	    QueryParser qp = new QueryParser("content", new StandardAnalyzer());
	    Query query = qp.parse(queryString);
	    Hits hits = searcher.search(query);


	    if(hits.length() > 0) {

		System.out.println("Hits for " + queryString + " were by by: ");

		for(int i = 0; i < hits.length(); i++) {

		    Document doc = hits.doc(i);
		    System.out.println(" " + (i + 1) + ". " + doc.get("filename"));

		}

	    }

	} catch(Exception e) {

	    e.printStackTrace();

	}

    }

    /**
     * Creates new instance
     */
    public LuceneSearcherImpl() {

    }

    /**
     * Runs the search given a query and a particular index
     *
     * @param String queryString             the query to be searched for
     * @param String idx                     the index to be searched
     * @return edu.arsc.multisearch.backend.ResultSet
     */
    public edu.arsc.multisearch.backend.lucene.webservice.ResultSetWritable search(String queryString) {
    //public ResultSetWritable search(String queryString) {

	//ResultSetWritable rsw = new ResultSetWritable(null, "LuceneSearcher via Axis", "Lucene indexing with no stopping or stemming");

	LinkedList allDocs = searchLucene(queryString);

	edu.arsc.multisearch.backend.lucene.webservice.DocumentWritable[] returnable = new edu.arsc.multisearch.backend.lucene.webservice.DocumentWritable[allDocs.size()];
	    
	    for(int z = 0; z < allDocs.size(); z++) {
		
		returnable[z] = (edu.arsc.multisearch.backend.lucene.webservice.DocumentWritable)allDocs.get(z);
		
	    }
	
	
	    /*
	      DocumentWritable[] returnable = new DocumentWritable[allDocs.size()];
	      
	      for(int z = 0; z < allDocs.size(); z++) {
	      
	      returnable[z] = (DocumentWritable)allDocs.get(z);
	      
	      }
	    */
	    edu.arsc.multisearch.backend.lucene.webservice.ResultSetWritable rsw = new edu.arsc.multisearch.backend.lucene.webservice.ResultSetWritable("Lucene 2.3.2 Regular indexing/searching, no stemming or stopping", "LuceneSearcher", returnable);
	    
	    //edu.arsc.multisearch.backend.lucene.webservice.ResultSet resSet = new edu.arsc.multisearch.backend.lucene.webservice.ResultSet("Lucene 2.3.2 Regular indexing/searching, no stemming or stopping", "LuceneSearcher", returnable);
	
	    
	    return(rsw);

	    //edu.arsc.multisearch.backend.lucene.webservice.ResultSetWritable rsw = new edu.arsc.multisearch.backend.lucene.webservice.ResultSetWritable("Lucene 2.3.2 Regular indexing/searching, no stemming or stopping", "LuceneSearcher@Fail", returnable);

	    //return(rsw);

    }

    /**
     * runs the search in Lucene
     */
    private LinkedList searchLucene(String queryString) {

	LinkedList allDocs = new LinkedList();


	try { 

	    Searcher searcher = new IndexSearcher(index);;
	    QueryParser qp = new QueryParser("content", new StandardAnalyzer());
	    Query query = qp.parse(queryString);
	    Hits hits = searcher.search(query);
	    	
	    if(hits.length() > 0) {
		
		for(int i = 0; i < hits.length(); i++) {
		    
		    Document doc = hits.doc(i);
		    
		    String title = "untitled";
		    if(doc.get("title") != null) {
			
			title = doc.get("title");
			
		    }
		    
		    //DocumentWritable docAdd = new DocumentWritable((i+1), title, hits.score(i), doc.get("filename"), "LuceneSearcher via Axis");
		    edu.arsc.multisearch.backend.lucene.webservice.DocumentWritable docAdd = new edu.arsc.multisearch.backend.lucene.webservice.DocumentWritable(doc.get("filename"), "LuceneSearcher", (i+1), hits.score(i), (i+1), hits.score(i), title);
		    
		    
		    allDocs.add(docAdd);
		    
		}
		
	    }
	    

	} catch(Exception e) {
	    
	    e.printStackTrace();
	    
	}

	return(allDocs);

    }

}
