/*
 * NAIVEMERGESET
 * The "final set" of ResultSets that are merged given
 * mapping to the Logistical Regression
 *
 * Some code taken from Aaron Luptak's 2005 code
 *
 * AUTHOR: McCormick, Kylie
 * DATE CREATED: 25 July 2006
 * LAST UPDATED: 7 July 2008
 * PROJECT: GridIR with Newby and Fallen
 *
 */
package edu.arsc.multisearch.merge;

import edu.arsc.multisearch.ResultSetWritable;
import java.util.LinkedList;
import edu.arsc.multisearch.DocumentWritable;
import edu.arsc.multisearch.OrderedList;

public class NaiveMergeSet extends FinalSet {


    /**
     * @return NaiveMergeSet
     */
    public NaiveMergeSet() {

	try {

	    results = null;
	    totalResults = 0;
	    rankedList = new OrderedList();

	} catch(Exception e) {

	    e.printStackTrace();
	}

    }

    /**
     * This function takes in a result set, rescores each result
     * and then adds it to the final ranked list at the end.
     *
     * NOTE: This code is from Aaron Luptak's 2005 Code
     *
     * @param DocumentWritable[]
     */
    public synchronized void merge(DocumentWritable[] hits) {

	//process results because of ranking
	hits = fromNeg(hits);

	int numresults = hits.length;
	double alpha;
	double beta;
	double sumX=0;
	double sumY=0; 
	double sumXY=0;
	double sumX2=0;
	    
	try {

	    for(int i = 0; i < numresults; i++) {
		
		double x = java.lang.Math.log((double)(i+1));
		double y = 0;

		DocumentWritable doc = hits[i];

		if(doc.getScore() > 0) {

		    double G = (double)(1-(doc.getScore() >= 1? .9999999 : doc.getScore())); 
		    G = (double)(doc.getScore()/G);
		    y = java.lang.Math.log(G);

		} 
		
		//sum the variables
		sumX += x;
		sumY += y;
		sumXY += (x * y);
		sumX2 += (x * x);

	    } //end for-each-doc


	    double D = sumX2*numresults - sumX*sumX;
	    beta = (sumXY*numresults - sumX*sumY)/D;
	    alpha = (sumX2*sumY - sumXY*sumX)/D;
	    
	    //This goes through each result and, using the stats above, calculates
	    //each result's new score
	    for(int x = 0; x < numresults; x++) {
		
		try {

		    DocumentWritable hit = hits[x];
		    double newscore = java.lang.Math.exp(alpha+beta*java.lang.Math.log((double)x+1))/(1+java.lang.Math.exp(alpha+beta*java.lang.Math.log((double)x+1)));

		    hit.setScore(newscore);
		    rankedList.addRanked(hit);

		    if(x == numresults-1) {
			
			rankedList.setRanks();
			totalResults = rankedList.size();
		    }


		} catch(Exception e) {
		
		    e.printStackTrace();
	
		}
		
	    }   

	    
	} catch(Exception e) {

	    e.printStackTrace();

	}

    }

    /**
     * Return the documents as a results
     *
     * @return DocumentWritable[]
     */
    public DocumentWritable[] getResults() {
	    
	DocumentWritable[] results = new DocumentWritable[rankedList.size()];
	
	for(int z = 0; z < rankedList.size(); z++) {
	    
	    results[z] = (DocumentWritable)rankedList.get(z);
	    
	}

	return(results);
	
    }

    /**
     * Takes negatively ranked documents and ranks them with positive numbers
     *
     * @param DocumentWritable[]
     * @return DocumentWritable[]
     */
    public DocumentWritable[] fromNeg(DocumentWritable[] hits) {

	try {

	    if(hits.length > 2) {

		double maxScore = hits[0].getScore();
		double minScore = hits[hits.length-1].getScore();
	    

		for(int i = 0; i < hits.length; i++) {

		    double score = hits[i].getScore();
		    score = score - minScore;
		    score = score / (maxScore - minScore);
		    hits[i].setScore(score);
		}

	    }


	} catch(Exception e) {

	    e.printStackTrace();

	}

	return(hits);
    }

}
