/**
 * An object used to search a given Lucene Index via Multisearch/Hadoop.
 *
 * @author Kylie McCormick
 * @project Multsiearch
 * @date 23 June 2008
 */
package edu.arsc.multisearch.backend.lucene;

import java.io.IOException;
import java.util.LinkedList;

import org.apache.lucene.search.Hits;
import org.apache.lucene.search.Query;
import org.apache.lucene.document.Field;
import org.apache.lucene.search.Searcher;
import org.apache.lucene.document.Document;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.analysis.standard.StandardAnalyzer;


public class LuceneSearcherImpl {

    
    /** The base URI of the files.**/
    public static String baseDir = "/home/mccormic/merge/indexes/";

    /**
     * Runs the search from the command line.
     *
     */
    public static void main(String args[]) {

	String query = "";
	String index = "";

	if(args.length > 0) {

	    index = args[0];

	    for(int x = 1; x < args.length; x++) {

		query += " " + args[x];

	    }

	    query = query.trim();
	}

	index = baseDir + index;

	
	searchit(query, index);

    }


    /**
     * Runs the search given a query and a particular index
     *
     * @param String queryString             the query to be searched for
     * @param String idx                     the index to be searched
     */
    public static void searchit(String queryString, String idx) {

	try {

	    Searcher searcher = new IndexSearcher(idx);;

	    QueryParser qp = new QueryParser("content", new StandardAnalyzer());
	    Query query = qp.parse(queryString);
	    Hits hits = searcher.search(query);


	    if(hits.length() > 0) {

		System.out.println("Hits for " + queryString + " were by by: ");

		for(int i = 0; i < hits.length(); i++) {

		    Document doc = hits.doc(i);
		    System.out.println(" " + (i + 1) + ". " + doc.get("filename"));

		}

	    }

	} catch(Exception e) {

	    e.printStackTrace();

	}

    }

    /**
     * Runs the search given a query and a particular index
     *
     * @param String queryString             the query to be searched for
     * @param String idx                     the index to be searched
     */
    public java.lang.Object[] search(String queryString, String idx) {

	try {

	    idx = baseDir + idx;

	    Searcher searcher = new IndexSearcher(idx);;

	    QueryParser qp = new QueryParser("content", new StandardAnalyzer());
	    Query query = qp.parse(queryString);
	    Hits hits = searcher.search(query);

	    LinkedList allDocs = new LinkedList();

	    if(hits.length() > 0) {

		for(int i = 0; i < hits.length(); i++) {

		    Document doc = hits.doc(i);

		    edu.arsc.multisearch.backend.Document docAdd = new edu.arsc.multisearch.backend.Document((i+1), doc.get("filename"), hits.score(i));

		    allDocs.add(docAdd);
		    //  allDocs.add(doc.get("filename"));

		}

	    }

	    java.lang.Object[] returnable = allDocs.toArray();

	    return(returnable);

	} catch(Exception e) {

	    e.printStackTrace();

	}


	return(null);

    }

}
