/**
 * A very simple document object for the backends of Multisearch.
 * Contains the bare minimum for adding a document to Multisearch.
 * <p>
 * Last Updated 28 July 2008
 * <p>
 * 
 * @author Kylie McCormick
 * @version 3.0
 *
 */
package edu.arsc.multisearch;

import java.io.DataInput;
import java.io.DataOutput;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.conf.*;
import org.apache.hadoop.io.*;
import org.apache.hadoop.mapred.*;
import org.apache.hadoop.util.*;
import java.util.StringTokenizer;

public class DocumentWritable implements Writable, Comparable {

    /** The title of the document */
    private String title = "untitled";

    /** The rank of the document. */
    private int rank;

   /** The score of the document */
    private double score;

    /** The filename of the document. */
    private String filename;

    /** The old rank of the document. */
    private int oldrank = 0;

    /** The old score of the document. */
    private double oldscore = 0.0;

    /** The backend name from which this document came. */
    private String name;

    /** The doc id associated with this document. */
    private String docid;
    
    /** The query number associated with this document as a result.*/
    private int queryNumber;

    /** The Label associated with this search/research.*/
    private String label;

    /**
     * @param DataInput
     */
    public void readFields(DataInput input) {

	try {

	    String line = input.readLine();
	    StringTokenizer st = new StringTokenizer(line, "=");

	    //Document Collection
	    while(st.hasMoreTokens()){

		// read in the variables from the line
		String rankString = st.nextToken();
		String oldrankString = st.nextToken();
		String titleString = st.nextToken();
		String scoreString = st.nextToken();
		String oldscoreString = st.nextToken();
		String filenameString = st.nextToken();
		String queryNumberString = st.nextToken();
		String labelString = st.nextToken();
		String nameString = st.nextToken();
		String docidString = st.nextToken();

		//parse them into the correct variations
		int rankI = Integer.parseInt(rankString);
		int oldrankI = Integer.parseInt(oldrankString);
		double scoreD = Double.parseDouble(scoreString);
		double oldscoreD = Double.parseDouble(oldscoreString);
		int queryNumberI = Integer.parseInt(queryNumberString);

		//set those variables!
		this.rank = rankI;
		this.oldrank = oldrankI;
		this.score = scoreD;
		this.oldscore = oldscoreD;
		this.filename = filenameString;
		this.title = titleString;
		this.queryNumber = queryNumberI;
		this.label = labelString;
		this.name = nameString;
		this.docid = docidString;

	    }

	} catch(java.io.IOException ioe) {

	    ioe.printStackTrace();

	}

    }

    /**
     * @param DataOutput
     */
    public void write(DataOutput output) {

	try {

	    if(title.equals("")) {

		title = "untitled";

	    }

	    if(filename.equals("")) {

		filename = "unknown";

	    }

	    if(label.equals("")) {

		label = "ANY";

	    }

	    if(docid.equals("")) {
		
		docid = "unknown";

	    }

	    if(name.equals("")) {

		name = "unknown";

	    }

	    String lineInput = rank + "=" +
		oldrank  + "=" +
		title + "=" +
		score + "=" +
		oldscore + "=" +
		filename + "=" +
		queryNumber + "=" +
		label + "=" +
		name + "=" +
		docid + "\n";
	    

	    output.writeBytes(lineInput);


	} catch(java.io.IOException ioe) {

	    ioe.printStackTrace();

	}

    }

    /**
     * Make a new Document object.
     * TREC variables will be initialized
     * 
     * @param int rank
     * @param String title
     * @param double score
     * @param String filename
     * @param String name
     */
    public DocumentWritable(int rank, int oldrank, String title, double score, double oldscore, String filename, String name) {

	this.rank = rank;
	this.oldrank = oldrank;
	this.title = title;
	this.score = score;
	this.oldscore = oldscore;
	this.filename = filename;
	this.name = name;

	this.queryNumber = 0;
	this.label = "ANY";
	
	StringTokenizer st = new StringTokenizer(filename, "//");
	String getit = "";

	while(st.hasMoreTokens()) {

	    getit = st.nextToken();

	}

	this.docid = getit.trim();
       
    }

    /**
     * Make a new Document object--the docid will be generated from filename
     * 
     * @param int rank
     * @param String title
     * @param double score
     * @param String filename
     * @param String name
     * @param int queryNumber
     * @param String label
     */
    public DocumentWritable(int rank, int oldrank, String title, double score, double oldscore, String filename, String name, int queryNumber, String label) {

	this.rank = rank;
	this.oldrank = oldrank;
	this.title = title;
	this.score = score;
	this.oldscore = oldscore;
	this.filename = filename;
	this.name = name;

	this.queryNumber = queryNumber;
	this.label = label;
	
	StringTokenizer st = new StringTokenizer(filename, "//");
	String getit = "";

	while(st.hasMoreTokens()) {

	    getit = st.nextToken();

	}

	this.docid = getit.trim();
       
    }

    /**
     * Make a new Document object.
     * 
     * @param int rank
     * @param int oldrank
     * @param String title
     * @param double score
     * @param double oldscore
     * @param String filename
     * @param String name
     * @param int queryNumber
     * @param String label
     * @param String docid
     */
    public DocumentWritable(int rank, int oldrank, String title, double score, double oldscore, String filename, String name, int queryNumber, String label, String docid) {

	this.rank = rank;
	this.oldrank = oldrank;
	this.title = title;
	this.score = score;
	this.oldscore = oldscore;
	this.filename = filename;
	this.name = name;
	this.queryNumber = queryNumber;
	this.label = label;
	this.docid = docid;
       
    }

    /**
     * Make a new Document bject.
     * 
     * @param int rank
     * @param String title
     * @param double score
     * @param String filename
     * @param String name
     */
    public DocumentWritable(int rank, String title, double score, String filename, String name) {

	this.rank = rank;
	this.oldrank = rank;
	this.title = title;
	this.score = score;
	this.oldscore = score;
	this.filename = filename;
	this.name = name;

	this.queryNumber = 0;
	this.label = "ANY";
	
	StringTokenizer st = new StringTokenizer(filename, "//");
	String getit = "";

	while(st.hasMoreTokens()) {

	    getit = st.nextToken();

	}

	this.docid = getit.trim();
       
    }

    /**
     * Make an empty document.
     *
     */
    public DocumentWritable() {

	rank = 0;
	oldrank = 0;
	title = "untitled";
	score = 0.0;
	oldscore = 0.00;
	filename = "";
	name = "";

    }

    /**
     * @param Object o
     * @return int
     */
    public int compareTo(Object o) {

	DocumentWritable compare = (DocumentWritable)o;

	if(compare.getScore() > score) {

	    return(-1); // less-than

	} else if(compare.getScore() < score) {

	    return(1); //greater-than

	} else {

	    return(0); //equal

	}

    }

    /**
     * @return String filename
     */
    public String getFilename() {

	return(filename);

    }

    /**
     * @return String title
     */
    public String getTitle() {

	return(title);

    }
    /**
     * @return int rank
     */
    public int getRank() {

	return(rank);

    }

    /**
     * @return double score
     */
    public double getScore() {

	return(score);

    }

    /**
     * @return double oldscore
     */
    public double getOldScore() {

	return(oldscore);

    }

    /**
     * @return int oldrank
     */
    public int getOldRank() {
	
	return(oldrank);

    }

    /**
     * @return String name
     */
    public String getName() {

	return(name);

    }

    /**
     * @param int rank
     */
    public void setRank(int rank) {

	this.rank = rank;

    }

    /**
     * @param double score
     */
    public void setScore(double score) {

	this.score = score;

    }

    /**
     * @param String label
     */
    public void setLabel(String label) {

	this.label = label;

    }

    /**
     * @param String docid
     */
    public void setDocID(String docid) {

	this.docid = docid;

    }

    /**
     * @param int queryNumber
     */
    public void setQueryNumber(int queryNumber) {

	this.queryNumber = queryNumber;

    }

    /**
     * @return String label
     */
    public String getLabel() {

	return(label);

    }

    /**
     * @return String docidc
     */
    public String getDocID() {

	return(docid);

    }

    /**
     * @return int queryNumber
     */
    public int getQueryNumber() {

	return(queryNumber);

    }

}
