/**
 * A class that defines the reduce (aka Merge) portion of 
 * Multisearch.
 *<p>
 * Last Updated: 31 July 2008
 *
 * @author Kylie McCormick
 * @version 3.0
 * @see org.apache.hadoop.mapred.Reducer
 */
package edu.arsc.multisearch;

import java.util.Iterator;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.conf.*;
import org.apache.hadoop.io.*;
import org.apache.hadoop.mapred.*;
import org.apache.hadoop.util.*;
import edu.arsc.multisearch.merge.*;

public class MergeReduce extends MapReduceBase implements Reducer<Text, ResultSetWritable, Text, ResultSetWritable> {

    /** Counters via Hadoop.*/
    static enum Counters {INPUT_SERVICES, OUTPUT_RESULTS};

    /**
     * The actual reduce function.
     *
     * @param Text text
     * @param Iterator<ServiceWritable> iterator
     * @param OutputCollector<Text, ResultSetWritable> outputCol
     * @param Reporter reporter
     */
    public void reduce(Text text, Iterator<ResultSetWritable> iterator, OutputCollector<Text, ResultSetWritable> outputCol, Reporter reporter) {

	try {

	    //create the appropriate kind of final set (naive merge for now)
	    String mergeClass = Multisearch.getMergeClass();
	    Class mergeMaker = Class.forName(mergeClass);
	    
	    //create the client
	    FinalSet set = (FinalSet)mergeMaker.newInstance();
	    
	    //iterate through the ServiceWritable, taking the output and merging it
	    while(iterator.hasNext()) {
		
		reporter.incrCounter(Counters.OUTPUT_RESULTS, 1);
		
		//grab the output result set
		ResultSetWritable rsw = iterator.next();
		DocumentWritable[] docs = rsw.get();
		
		reporter.setStatus("Current input from iterator: " + rsw.getName() + " with key " + text.toString());
		
		if(docs != null) {
		    
		    //merge them together
		    set.merge(docs);
		    
		} 

	    } //end-while-has-iterator
	    
	    
	    ResultSetWritable rsw = new ResultSetWritable(set.getResults(), "FinalSet", "FinalSet for Multisearch");
	    Text newKey = new Text(Multisearch.getQuery());
	    
	    //collect the merged function
	    outputCol.collect(newKey, rsw);
	    
	} catch(java.lang.ClassNotFoundException cnfe) {
	    
	    reporter.setStatus(new String("Class declared for Merge NOT FOUND! " + cnfe.toString()));

	} catch(java.lang.InstantiationException ie) {

	    reporter.setStatus(new String("Class declared for Merge cannot be instatiated. " + ie.toString()));

	} catch(java.lang.IllegalAccessException iae) {

	    reporter.setStatus(new String("Class declared for Merge is illegal." + iae.toString()));

	} catch(java.io.IOException ioe) {
	    
	    reporter.setStatus(ioe.toString());
	    
	} 
	
	 
    }

}





