/**
 * An object to serialize a call between Multisearch and an Axis Service.
 * 
 * @author Kylie McCormick
 * @date 17 July 2008
 * @update 18 July 2008
 */
package edu.arsc.multisearch;

import java.io.*;
import java.util.*;
import javax.xml.*;
import org.apache.axis.soap.SOAPConnectionImpl;
import org.apache.axis.client.Call;
import org.apache.axis.client.Response;
import org.apache.axis.client.Service;
import org.apache.axis.encoding.XMLType;
import org.apache.axis.utils.Options;
import org.apache.axis.client.*;
import org.apache.axis.Message;
import javax.xml.namespace.QName;
import org.apache.axis.encoding.ser.BeanSerializerFactory;
import org.apache.axis.encoding.ser.BeanDeserializerFactory;

public class AxisClient extends Client {


    /**
     * Call the query on the backend.
     *
     * @param String query
     */
    public void call(String query) {

	rsw = new ResultSetWritable();

	try {

	    Service service = new Service();
	    Call call = (Call)service.createCall();

	    //Let's make sure that the call knows how to deserialize the 
	    //given WebService items, in thisc ase LuceneBackend: ResultSet
	    QName qNameRS = new QName("urn:LuceneSeacher", "ResultSet");
	    BeanSerializerFactory bsfRS = new BeanSerializerFactory(edu.arsc.multisearch.backend.lucene.webservice.ResultSet.class, qNameRS);
	    BeanDeserializerFactory bdsfRS = new BeanDeserializerFactory(edu.arsc.multisearch.backend.lucene.webservice.ResultSet.class, qNameRS);
	    call.registerTypeMapping(edu.arsc.multisearch.backend.lucene.webservice.ResultSet.class, qNameRS, bsfRS, bdsfRS);
	    

	    //Making the call work!
	    call.setTargetEndpointAddress(new java.net.URL(URL));
	    call.setOperationName(new QName("urn:LuceneSearcher", "search")); //Name of the function in LUCENEBACKEND
	    call.setReturnClass(edu.arsc.multisearch.backend.lucene.webservice.ResultSet.class);
	    call.setReturnType(qNameRS);
	    call.setUseSOAPAction(true);
	    call.setEncodingStyle(org.apache.axis.Constants.URI_SOAP11_ENC);
	    call.setSOAPVersion(org.apache.axis.soap.SOAPConstants.SOAP11_CONSTANTS);
	    call.addParameter("in0", XMLType.XSD_STRING, javax.xml.rpc.ParameterMode.IN);
	    call.addParameter("rss", qNameRS, edu.arsc.multisearch.backend.lucene.webservice.ResultSet.class, javax.xml.rpc.ParameterMode.OUT);

	    String[] params = new String[1];
	    params[0] = query;
	    
	    //Response response = call.invoke(params);
	  
	    edu.arsc.multisearch.backend.lucene.webservice.ResultSet wsResultSet = (edu.arsc.multisearch.backend.lucene.webservice.ResultSet)call.invoke(params);


	    edu.arsc.multisearch.backend.lucene.webservice.Document[] documents = wsResultSet.getResults();
	    if(documents == null) {

		throw new java.lang.NullPointerException("Documents null on arrival from " + wsResultSet.getName());
	    }


	    Document[] xDocs = new Document[documents.length];

	    for(int q = 0; q < documents.length; q++) {

		xDocs[q] = new Document(documents[q].getRank(), 
					documents[q].getTitle(),
					documents[q].getScore(),
					documents[q].getFilename(),
					documents[q].getName());

	    }

	    rsw.setResults(xDocs);
	    rsw.setInfo(wsResultSet.getInfo());
	    rsw.setName(wsResultSet.getName());


	} catch(Exception e) {

 	    e.printStackTrace();

	}

    }
  
}
