/**
 * Taken in part from Multisearch 2007, this Client opens a LuceneSearchToolkit
 * and runs it for Multisearch/Hadoop 2008. This is for <b>OGSA-DAI</b> Clients
 * and is built on OGSA-DAI 2.2 from Summer 2007.
 * <p>
 * Many of the items here were instances declared by Multisearch 2007 which are
 * not needed for Multisearch 2008 (such as min/max) but are still needed to 
 * work with OGSA-DAI backends from 2007.
 * <p>
 * Last Updated: 28 July
 * <p>
 *
 * @author Kylie McCormick
 * @version 3.0
 */
package edu.arsc.multisearch.client;

import edu.arsc.multisearch.*;
import java.io.File;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.LinkedList;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import uk.org.ogsadai.client.toolkit.GenericServiceFetcher;
import uk.org.ogsadai.client.toolkit.ResourceID;
import uk.org.ogsadai.client.toolkit.service.DataService;
import uk.org.ogsadai.client.toolkit.GenericServiceFetcher;
import uk.org.ogsadai.client.toolkit.Response;
import uk.org.ogsadai.client.toolkit.activity.ActivityRequest;
import uk.org.ogsadai.client.toolkit.activity.ActivityOutput;
import uk.org.ogsadai.client.toolkit.activity.indexedfiles.IndexedFileSearch;
import uk.org.ogsadai.client.toolkit.activity.indexedfiles.ReadFile;
import uk.org.ogsadai.common.msgs.DAILogger;
import edu.arsc.multisearch.merge.*;
import edu.arsc.multisearch.ogsadai.*;
import uk.org.ogsadai.client.toolkit.activity.Activity;
import uk.org.ogsadai.exception.DAIException;
import uk.org.ogsadai.client.toolkit.ResourceID;
import java.lang.Thread;

public class LuceneDaiClient extends Client {

    /** Minimum number of results*/
    private int max = 100;

    /** Maximum number of results*/
    private int min = 5;

    /** The research toolkit */
    private SearchToolkit lst;

    private String classname = "edu.arsc.multisearch.client.LuceneSearchToolkit";

    private String rIndex = "DEFAULT";
    
    /**
     * The thread opens up a connection to the given Data Service,
     * creates an activity, attaches its Toolkit, and sents a perform
     * document. When the results are back, it connects them to the
     * ResourceBlock.
     */
    public void call(String query) {

	try {

	    Class toolkit = Class.forName(classname);
	    lst = (SearchToolkit)toolkit.newInstance();
            lst.setMin(min);
            lst.setMax(max);
	    lst.setQuery(query);
            lst.setIndex(rIndex);

	    //make the service connection
	    String handle = URL;
	    handle = handle.trim();
	    String rID = name;
	    rID = rID.trim();

	    //open data services, and full steam ahead!
	    DataService service = GenericServiceFetcher.getInstance().getDataService(handle, rID);  
	    ActivityRequest request = new ActivityRequest();
	    request.add(lst);
	    Response response = service.perform(request);

	    //Get the data in a usable form
	    ActivityOutput outputData = lst.getSearchStatus();
	    String cData = outputData.getData();

	    //Generate A Linked List
	    DocumentWritable[] results = makeDocs(cData, name, name);

	    //save it to the resource, eh?
	    rsw = new ResultSetWritable(results, name, name);

	    //System.out.println("There are " + results.length + " results.");

        } catch(DAIException e) {

	    e.printStackTrace();
	    
	} catch(Exception e) {
	    
	    e.printStackTrace();

	}
      

    }

    /**
     * Turns XML responses into Document Objects that are
     * returned in an Array.
     *
     * @param String     the XML CDATA response string
     * @param String     the DataResourceName
     * @return           of Document objects
      */
    private DocumentWritable[] makeDocs(String rS, String name, String rName) {

	LinkedList responseList = new LinkedList();
       	File xmlfile = makeXMLFile(name, rName);

	if(xmlfile != null) {

	    try {

		java.io.FileOutputStream fos = new java.io.FileOutputStream(xmlfile);
		java.io.PrintStream printXML = new java.io.PrintStream(fos);

		printXML.print("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
		printXML.println(rS);
	    
		//now we can treat it like an XML document
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		org.w3c.dom.Document response = factory.newDocumentBuilder().parse(xmlfile); 

		//processing...
		Element element = response.getDocumentElement();
		NodeList nodes = element.getElementsByTagName("document");
	
		if(nodes != null && nodes.getLength() > 0) {
		    
		    for(int d = 0; d < nodes.getLength(); d++) {

			//grab a node
			Element node = (Element)nodes.item(d);

			//change the string numbers into values
			String score = node.getAttribute("score");
			String rank = node.getAttribute("rank");
			String filePath = URL + node.getAttribute("filename");
			String title = node.getAttribute("title");

			if(title == null || title.equals("")) {
			
			    title = "Untitled";
			}

			//int for rank
			java.lang.Integer integerX = new java.lang.Integer(rank);
			int rankIt = integerX.intValue();

			//double for score
			java.lang.Double doubleX = new java.lang.Double(score);
			double scoreIt = doubleX.doubleValue();

			DocumentWritable newDoc = new DocumentWritable(rankIt, rankIt, title, scoreIt, scoreIt, filePath, this.name, Multisearch.getQueryNumber(), Multisearch.getLabel(), node.getAttribute("id"));
		    
			responseList.add(newDoc);
		    }
		
		} 
	    
		//clean up XML file
		xmlfile.delete();

	    } catch(java.io.IOException e) {

		e.printStackTrace();

	    } catch(javax.xml.parsers.ParserConfigurationException e) {

		e.printStackTrace();

	    } catch(org.xml.sax.SAXException e) {
		
		e.printStackTrace();

	    }

	}

	    DocumentWritable[] resultsHere = new DocumentWritable[responseList.size()];

	    for(int i = 0; i < responseList.size(); i++) {
		
		resultsHere[i] = (DocumentWritable)responseList.get(i);		    
		
	    }
	    
	    
	    
	

	return(resultsHere);

    }

    /**
     * A function to create an XML file to be used by the parser.
     *
     * @param String        name of the server
     * @param String        name of the resource
     * @return File 
     */
    private File makeXMLFile(String server, String resource) {

	File xmlfile = null;
	
	try {

	    String uName = server.replace(" ", "");
	    uName = uName.replace(":", "");
	    uName = uName.trim();
	    
	    String urName = resource.replace(" ", "");
	    urName = urName.trim();
	    
	    //unique xmlfile name string
	    String xmlFileName = uName + "_" + urName + ".xml";

  	    xmlfile = File.createTempFile("output_", xmlFileName);

	} catch(java.io.IOException e) {

	    e.printStackTrace();

	}

	return(xmlfile);

    }

}
